@@ -6,7 +6,7 @@
 //
 // Copyright 2015, 2016 André Guerreiro <aguerreiro1985@gmail.com>
 // Copyright 2015 André Esser <bepandre@hotmail.com>
-// Copyright 2015, 2016, 2018, 2019, 2021, 2022 Albert Astals Cid <aacid@kde.org>
+// Copyright 2015, 2016, 2018, 2019, 2021-2023 Albert Astals Cid <aacid@kde.org>
 // Copyright 2015 Markus Kilås <digital@markuspage.com>
 // Copyright 2017 Sebastian Rasmussen <sebras@gmail.com>
 // Copyright 2017 Hans-Ulrich Jüttner <huj@froreich-bioscientia.de>
@@ -18,6 +18,8 @@
 // Copyright 2021 Marek Kasik <mkasik@redhat.com>
 // Copyright 2022 Erich E. Hoover <erich.e.hoover@gmail.com>
 // Copyright 2023 Tobias Deiminger <tobias.deiminger@posteo.de>
+// Copyright 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright 2023 Ingo Klöcker <kloecker@kde.org>
 //
 //========================================================================
 
@@ -28,6 +30,7 @@
 #include "goo/gmem.h"
 
 #include <optional>
+#include <vector>
 
 #include <Error.h>
 
@@ -46,70 +49,6 @@
 #include <cms.h>
 #include <cmst.h>
 
-// ASN.1 used in the (much simpler) time stamp request. From RFC3161
-// and other sources.
-
-/*
-AlgorithmIdentifier  ::=  SEQUENCE  {
-     algorithm  OBJECT IDENTIFIER,
-     parameters ANY DEFINED BY algorithm OPTIONAL  }
-                   -- contains a value of the type
-                   -- registered for use with the
-                   -- algorithm object identifier value
-
-MessageImprint ::= SEQUENCE  {
-    hashAlgorithm AlgorithmIdentifier,
-    hashedMessage OCTET STRING  }
-*/
-
-struct MessageImprint
-{
-    SECAlgorithmID hashAlgorithm;
-    SECItem hashedMessage;
-};
-
-/*
-Extension  ::=  SEQUENCE  {
-    extnID    OBJECT IDENTIFIER,
-    critical  BOOLEAN DEFAULT FALSE,
-    extnValue OCTET STRING  }
-*/
-
-struct Extension
-{
-    SECItem const extnID;
-    SECItem const critical;
-    SECItem const extnValue;
-};
-
-/*
-Extensions ::= SEQUENCE SIZE (1..MAX) OF Extension
-*/
-
-/*
-TSAPolicyId ::= OBJECT IDENTIFIER
-
-TimeStampReq ::= SEQUENCE  {
-    version            INTEGER  { v1(1) },
-    messageImprint     MessageImprint,
-    --a hash algorithm OID and the hash value of the data to be
-    --time-stamped
-    reqPolicy          TSAPolicyId         OPTIONAL,
-    nonce              INTEGER             OPTIONAL,
-    certReq            BOOLEAN             DEFAULT FALSE,
-    extensions     [0] IMPLICIT Extensions OPTIONAL  }
-*/
-
-struct TimeStampReq
-{
-    SECItem version;
-    MessageImprint messageImprint;
-    SECItem reqPolicy;
-    SECItem nonce;
-    SECItem certReq;
-    Extension *extensions;
-};
-
 /**
  * General name, defined by RFC 3280.
  */
@@ -258,6 +197,10 @@ const SEC_ASN1Template TimeStampReq_Template[] = { { SEC_ASN1_SEQUENCE, 0, nullp
                                                    { 0, 0, nullptr, 0 } };
 */
 
+static NSSCMSMessage *CMS_MessageCreate(SECItem *cms_item);
+static NSSCMSSignedData *CMS_SignedDataCreate(NSSCMSMessage *cms_msg);
+static NSSCMSSignerInfo *CMS_SignerInfoCreate(NSSCMSSignedData *cms_sig_data);
+
 // a dummy, actually
 static char *passwordCallback(PK11SlotInfo * /*slot*/, PRBool /*retry*/, void *arg)
 {
@@ -274,7 +217,7 @@ static void shutdownNss()
 // SEC_StringToOID() and NSS_CMSSignerInfo_AddUnauthAttr() are
 // not exported from libsmime, so copy them here. Sigh.
 
-static SECStatus my_SEC_StringToOID(SECItem *to, const char *from, PRUint32 len)
+static SECStatus my_SEC_StringToOID(PLArenaPool *arena, SECItem *to, const char *from, PRUint32 len)
 {
     PRUint32 decimal_numbers = 0;
     PRUint32 result_bytes = 0;
@@ -367,7 +310,7 @@ static SECStatus my_SEC_StringToOID(SECItem *to, const char *from, PRUint32 len)
         SECItem result_item = { siBuffer, nullptr, 0 };
         result_item.data = result;
         result_item.len = result_bytes;
-        rv = SECITEM_CopyItem(nullptr, to, &result_item);
+        rv = SECITEM_CopyItem(arena, to, &result_item);
     }
     return rv;
 }
@@ -491,16 +434,63 @@ static SECStatus my_NSS_CMSSignerInfo_AddAuthAttr(NSSCMSSignerInfo *signerinfo,
     return my_NSS_CMSAttributeArray_AddAttr(signerinfo->cmsg->poolp, &(signerinfo->authAttr), attr);
 }
 
-unsigned int SignatureHandler::digestLength(SECOidTag digestAlgId)
+static SECOidTag ConvertHashAlgorithmToNss(HashAlgorithm digestAlgId)
 {
     switch (digestAlgId) {
-    case SEC_OID_SHA1:
+    case HashAlgorithm::Md2:
+        return SEC_OID_MD2;
+    case HashAlgorithm::Md5:
+        return SEC_OID_MD5;
+    case HashAlgorithm::Sha1:
+        return SEC_OID_SHA1;
+    case HashAlgorithm::Sha256:
+        return SEC_OID_SHA256;
+    case HashAlgorithm::Sha384:
+        return SEC_OID_SHA384;
+    case HashAlgorithm::Sha512:
+        return SEC_OID_SHA512;
+    case HashAlgorithm::Sha224:
+        return SEC_OID_SHA224;
+    case HashAlgorithm::Unknown:
+        return SEC_OID_UNKNOWN;
+    }
+    return SEC_OID_UNKNOWN;
+}
+
+static HashAlgorithm ConvertHashTypeFromNss(HASH_HashType type)
+{
+    switch (type) {
+    case HASH_AlgMD2:
+        return HashAlgorithm::Md2;
+    case HASH_AlgMD5:
+        return HashAlgorithm::Md5;
+    case HASH_AlgSHA1:
+        return HashAlgorithm::Sha1;
+    case HASH_AlgSHA256:
+        return HashAlgorithm::Sha256;
+    case HASH_AlgSHA384:
+        return HashAlgorithm::Sha384;
+    case HASH_AlgSHA512:
+        return HashAlgorithm::Sha512;
+    case HASH_AlgSHA224:
+        return HashAlgorithm::Sha224;
+    case HASH_AlgNULL:
+    case HASH_AlgTOTAL:
+        return HashAlgorithm::Unknown;
+    }
+    return HashAlgorithm::Unknown;
+}
+
+static unsigned int digestLength(HashAlgorithm digestAlgId)
+{
+    switch (digestAlgId) {
+    case HashAlgorithm::Sha1:
         return 20;
-    case SEC_OID_SHA256:
+    case HashAlgorithm::Sha256:
         return 32;
-    case SEC_OID_SHA384:
+    case HashAlgorithm::Sha384:
         return 48;
-    case SEC_OID_SHA512:
+    case HashAlgorithm::Sha512:
         return 64;
     default:
         printf("ERROR: Unrecognized Hash ID\n");
@@ -508,42 +498,21 @@ unsigned int SignatureHandler::digestLength(SECOidTag digestAlgId)
     }
 }
 
-SECOidTag SignatureHandler::getHashOidTag(const char *digestName)
+std::string SignatureVerificationHandler::getSignerName() const
 {
-    SECOidTag tag = SEC_OID_UNKNOWN;
-    if (strcmp(digestName, "SHA1") == 0) {
-        tag = SEC_OID_SHA1;
-    } else if (strcmp(digestName, "SHA256") == 0) {
-        tag = SEC_OID_SHA256;
-    } else if (strcmp(digestName, "SHA384") == 0) {
-        tag = SEC_OID_SHA384;
-    } else if (strcmp(digestName, "SHA512") == 0) {
-        tag = SEC_OID_SHA512;
-    }
-    return tag;
-}
-
-std::string SignatureHandler::getSignerName()
-{
-    char *commonName;
-
     if (!NSS_IsInitialized()) {
         return {};
     }
-
-    if (!signing_cert && !CMSSignerInfo) {
+    if (!CMSSignerInfo) {
         return {};
     }
 
-    if (!signing_cert) {
-        signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
-    }
-
+    auto signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
     if (!signing_cert) {
         return {};
     }
 
-    commonName = CERT_GetCommonName(&signing_cert->subject);
+    char *commonName = CERT_GetCommonName(&signing_cert->subject);
     if (!commonName) {
         return {};
     }
@@ -553,33 +522,23 @@ std::string SignatureHandler::getSignerName()
     return name;
 }
 
-const char *SignatureHandler::getSignerSubjectDN()
+std::string SignatureVerificationHandler::getSignerSubjectDN() const
 {
-    if (!signing_cert && !CMSSignerInfo) {
-        return nullptr;
+    if (!CMSSignerInfo) {
+        return {};
     }
-
+    auto signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
     if (!signing_cert) {
-        signing_cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
+        return {};
     }
-
-    if (!signing_cert) {
-        return nullptr;
-    }
-
-    return signing_cert->subjectName;
+    return std::string { signing_cert->subjectName };
 }
 
-HASH_HashType SignatureHandler::getHashAlgorithm()
+time_t SignatureVerificationHandler::getSigningTime() const
 {
-    if (hash_context && hash_context->hashobj) {
-        return hash_context->hashobj->type;
+    if (!CMSSignerInfo) {
+        return {};
     }
-    return HASH_AlgNULL;
-}
-
-time_t SignatureHandler::getSigningTime()
-{
     PRTime sTime; // time in microseconds since the epoch
 
     if (NSS_CMSSignerInfo_GetSigningTime(CMSSignerInfo, &sTime) != SECSuccess) {
@@ -687,21 +646,24 @@ static std::unique_ptr<X509CertificateInfo> getCertificateInfoFromCERT(CERTCerti
     return certInfo;
 }
 
-std::unique_ptr<X509CertificateInfo> SignatureHandler::getCertificateInfo() const
+std::unique_ptr<X509CertificateInfo> SignatureVerificationHandler::getCertificateInfo() const
 {
-    if (CMSSignerInfo) {
-        CERTCertificate *cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
-        if (!cert) {
-            return nullptr;
-        }
-        return getCertificateInfoFromCERT(cert);
-    } else {
-        if (!signing_cert) {
-            return nullptr;
-        }
-
-        return getCertificateInfoFromCERT(signing_cert);
+    if (!CMSSignerInfo) {
+        return nullptr;
     }
+    CERTCertificate *cert = NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB());
+    if (!cert) {
+        return nullptr;
+    }
+    return getCertificateInfoFromCERT(cert);
+}
+
+std::unique_ptr<X509CertificateInfo> SignatureSignHandler::getCertificateInfo() const
+{
+    if (!signing_cert) {
+        return nullptr;
+    }
+    return getCertificateInfoFromCERT(signing_cert);
 }
 
 static std::optional<std::string> getDefaultFirefoxCertDB()
@@ -794,78 +756,75 @@ void SignatureHandler::setNSSPasswordCallback(const std::function<char *(const c
     PasswordFunction = f;
 }
 
-SignatureHandler::SignatureHandler(unsigned char *p7, int p7_length) : hash_context(nullptr), CMSMessage(nullptr), CMSSignedData(nullptr), CMSSignerInfo(nullptr), signing_cert(nullptr), temp_certs(nullptr)
+SignatureVerificationHandler::SignatureVerificationHandler(std::vector<unsigned char> &&p7data) : p7(std::move(p7data)), CMSMessage(nullptr), CMSSignedData(nullptr), CMSSignerInfo(nullptr)
 {
-    setNSSDir({});
-    CMSitem.data = p7;
-    CMSitem.len = p7_length;
+    SignatureHandler::setNSSDir({});
+    CMSitem.data = p7.data();
+    CMSitem.len = p7.size();
     CMSMessage = CMS_MessageCreate(&CMSitem);
     CMSSignedData = CMS_SignedDataCreate(CMSMessage);
     if (CMSSignedData) {
         CMSSignerInfo = CMS_SignerInfoCreate(CMSSignedData);
-        hash_context = initHashContext();
+        SECItem usedAlgorithm = NSS_CMSSignedData_GetDigestAlgs(CMSSignedData)[0]->algorithm;
+        auto hashAlgorithm = SECOID_FindOIDTag(&usedAlgorithm);
+        HASH_HashType hashType = HASH_GetHashTypeByOidTag(hashAlgorithm);
+        hashContext = std::make_unique<HashContext>(ConvertHashTypeFromNss(hashType));
     }
 }
 
-SignatureHandler::SignatureHandler(const char *certNickname, SECOidTag digestAlgTag)
-    : hash_length(digestLength(digestAlgTag)), digest_alg_tag(digestAlgTag), CMSitem(), hash_context(nullptr), CMSMessage(nullptr), CMSSignedData(nullptr), CMSSignerInfo(nullptr), signing_cert(nullptr), temp_certs(nullptr)
+SignatureSignHandler::SignatureSignHandler(const std::string &certNickname, HashAlgorithm digestAlgTag) : hashContext(std::make_unique<HashContext>(digestAlgTag)), signing_cert(nullptr)
 {
-    setNSSDir({});
-    CMSMessage = NSS_CMSMessage_Create(nullptr);
-    signing_cert = CERT_FindCertByNickname(CERT_GetDefaultCertDB(), certNickname);
-    hash_context = HASH_Create(HASH_GetHashTypeByOidTag(digestAlgTag));
+    SignatureHandler::setNSSDir({});
+    signing_cert = CERT_FindCertByNickname(CERT_GetDefaultCertDB(), certNickname.c_str());
 }
 
-SignatureHandler::SignatureHandler() : hash_length(), digest_alg_tag(), CMSitem(), hash_context(nullptr), CMSMessage(nullptr), CMSSignedData(nullptr), CMSSignerInfo(nullptr), signing_cert(nullptr), temp_certs(nullptr)
+HashAlgorithm SignatureVerificationHandler::getHashAlgorithm() const
 {
-    setNSSDir({});
-    CMSMessage = NSS_CMSMessage_Create(nullptr);
-}
-
-HASHContext *SignatureHandler::initHashContext()
-{
-
-    SECItem usedAlgorithm = NSS_CMSSignedData_GetDigestAlgs(CMSSignedData)[0]->algorithm;
-    hash_length = digestLength(SECOID_FindOIDTag(&usedAlgorithm));
-    HASH_HashType hashType;
-    hashType = HASH_GetHashTypeByOidTag(SECOID_FindOIDTag(&usedAlgorithm));
-    return HASH_Create(hashType);
-}
-
-void SignatureHandler::updateHash(unsigned char *data_block, int data_len)
-{
-    if (hash_context) {
-        HASH_Update(hash_context, data_block, data_len);
+    if (hashContext) {
+        return hashContext->getHashAlgorithm();
+    } else {
+        return HashAlgorithm::Unknown;
     }
 }
 
-void SignatureHandler::restartHash()
+void SignatureVerificationHandler::updateHash(unsigned char *data_block, int data_len)
 {
-    if (hash_context) {
-        HASH_Destroy(hash_context);
+    if (hashContext) {
+        hashContext->updateHash(data_block, data_len);
     }
-    hash_context = HASH_Create(HASH_GetHashTypeByOidTag(digest_alg_tag));
 }
 
-SignatureHandler::~SignatureHandler()
+void SignatureSignHandler::updateHash(unsigned char *data_block, int data_len)
 {
-    SECITEM_FreeItem(&CMSitem, PR_FALSE);
-    if (CMSMessage) {
-        NSS_CMSMessage_Destroy(CMSMessage);
-    }
-
-    if (hash_context) {
-        HASH_Destroy(hash_context);
-    }
+    hashContext->updateHash(data_block, data_len);
+}
 
+SignatureSignHandler::~SignatureSignHandler()
+{
     if (signing_cert) {
         CERT_DestroyCertificate(signing_cert);
     }
-
-    free(temp_certs);
+}
+SignatureVerificationHandler::~SignatureVerificationHandler()
+{
+    if (CMSMessage) {
+        // in the CMS_SignedDataCreate, we malloc some memory
+        // inside the CMSSignedData structure
+        // which is otherwise destructed by NSS_CMSMessage_Destroy
+        // but given we did the malloc ourselves
+        // we also need to free it ourselves.
+        // After we free the surrounding memory but we need
+        // a handle to it before.
+        CERTCertificate **toFree = nullptr;
+        if (CMSSignedData) {
+            toFree = CMSSignedData->tempCerts;
+        }
+        NSS_CMSMessage_Destroy(CMSMessage);
+        free(toFree);
+    }
 }
 
-NSSCMSMessage *SignatureHandler::CMS_MessageCreate(SECItem *cms_item)
+static NSSCMSMessage *CMS_MessageCreate(SECItem *cms_item)
 {
     if (cms_item->data) {
         return NSS_CMSMessage_CreateFromDER(cms_item, nullptr, nullptr /* Content callback */
@@ -878,7 +837,7 @@ NSSCMSMessage *SignatureHandler::CMS_MessageCreate(SECItem *cms_item)
     }
 }
 
-NSSCMSSignedData *SignatureHandler::CMS_SignedDataCreate(NSSCMSMessage *cms_msg)
+static NSSCMSSignedData *CMS_SignedDataCreate(NSSCMSMessage *cms_msg)
 {
     if (!NSS_CMSMessage_IsSigned(cms_msg)) {
         error(errInternal, 0, "Input couldn't be parsed as a CMS signature");
@@ -908,15 +867,13 @@ NSSCMSSignedData *SignatureHandler::CMS_SignedDataCreate(NSSCMSMessage *cms_msg)
         for (i = 0; signedData->rawCerts[i]; ++i) {
             signedData->tempCerts[i] = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), signedData->rawCerts[i], nullptr, 0, 0);
         }
-
-        temp_certs = signedData->tempCerts;
         return signedData;
     } else {
         return nullptr;
     }
 }
 
-NSSCMSSignerInfo *SignatureHandler::CMS_SignerInfoCreate(NSSCMSSignedData *cms_sig_data)
+static NSSCMSSignerInfo *CMS_SignerInfoCreate(NSSCMSSignedData *cms_sig_data)
 {
     NSSCMSSignerInfo *signerInfo = NSS_CMSSignedData_GetSignerInfo(cms_sig_data, 0);
     if (!signerInfo) {
@@ -947,10 +904,8 @@ static SignatureValidationStatus NSS_SigTranslate(NSSCMSVerificationStatus nss_c
     }
 }
 
-SignatureValidationStatus SignatureHandler::validateSignature()
+SignatureValidationStatus SignatureVerificationHandler::validateSignature()
 {
-    unsigned char *digest_buffer = nullptr;
-
     if (!CMSSignedData) {
         return SIGNATURE_GENERIC_ERROR;
     }
@@ -959,18 +914,15 @@ SignatureValidationStatus SignatureHandler::validateSignature()
         return SIGNATURE_GENERIC_ERROR;
     }
 
-    if (!hash_context) {
+    if (!hashContext) {
         return SIGNATURE_GENERIC_ERROR;
     }
 
-    digest_buffer = (unsigned char *)PORT_Alloc(hash_length);
-    unsigned int result_len = 0;
-
-    HASH_End(hash_context, digest_buffer, &result_len, hash_length);
+    std::vector<unsigned char> digest_buffer = hashContext->endHash();
 
     SECItem digest;
-    digest.data = digest_buffer;
-    digest.len = hash_length;
+    digest.data = digest_buffer.data();
+    digest.len = digest_buffer.size();
 
     if ((NSS_CMSSignerInfo_GetSigningCertificate(CMSSignerInfo, CERT_GetDefaultCertDB())) == nullptr) {
         CMSSignerInfo->verificationStatus = NSSCMSVS_SigningCertNotFound;
@@ -982,25 +934,20 @@ SignatureValidationStatus SignatureHandler::validateSignature()
           This means it's not a detached type signature
           so the digest is contained in SignedData->contentInfo
         */
-        if (memcmp(digest.data, content_info_data->data, hash_length) == 0 && digest.len == content_info_data->len) {
-            PORT_Free(digest_buffer);
+        if (digest.len == content_info_data->len && memcmp(digest.data, content_info_data->data, digest.len) == 0) {
             return SIGNATURE_VALID;
         } else {
-            PORT_Free(digest_buffer);
             return SIGNATURE_DIGEST_MISMATCH;
         }
 
     } else if (NSS_CMSSignerInfo_Verify(CMSSignerInfo, &digest, nullptr) != SECSuccess) {
-
-        PORT_Free(digest_buffer);
         return NSS_SigTranslate(CMSSignerInfo->verificationStatus);
     } else {
-        PORT_Free(digest_buffer);
         return SIGNATURE_VALID;
     }
 }
 
-CertificateValidationStatus SignatureHandler::validateCertificate(time_t validation_time, bool ocspRevocationCheck, bool useAIACertFetch)
+CertificateValidationStatus SignatureVerificationHandler::validateCertificate(time_t validation_time, bool ocspRevocationCheck, bool useAIACertFetch)
 {
     CERTCertificate *cert;
 
@@ -1056,47 +1003,48 @@ CertificateValidationStatus SignatureHandler::validateCertificate(time_t validat
     return CERTIFICATE_GENERIC_ERROR;
 }
 
-std::unique_ptr<GooString> SignatureHandler::signDetached(const char *password) const
+std::unique_ptr<GooString> SignatureSignHandler::signDetached(const std::string &password)
 {
-    if (!hash_context) {
+    if (!hashContext) {
         return nullptr;
     }
-    unsigned char *digest_buffer = reinterpret_cast<unsigned char *>(PORT_Alloc(hash_length));
-    unsigned int result_len = 0;
-    HASH_End(hash_context, digest_buffer, &result_len, hash_length);
+    std::vector<unsigned char> digest_buffer = hashContext->endHash();
     SECItem digest;
-    digest.data = digest_buffer;
-    digest.len = result_len;
+    digest.data = digest_buffer.data();
+    digest.len = digest_buffer.size();
 
     /////////////////////////////////////
     /// Code from LibreOffice under MPLv2
     /////////////////////////////////////
-
-    NSSCMSMessage *cms_msg = NSS_CMSMessage_Create(nullptr);
+    struct NSSCMSMessageDestroyer
+    {
+        void operator()(NSSCMSMessage *message) { NSS_CMSMessage_Destroy(message); }
+    };
+    std::unique_ptr<NSSCMSMessage, NSSCMSMessageDestroyer> cms_msg { NSS_CMSMessage_Create(nullptr) };
     if (!cms_msg) {
         return nullptr;
     }
 
-    NSSCMSSignedData *cms_sd = NSS_CMSSignedData_Create(cms_msg);
+    NSSCMSSignedData *cms_sd = NSS_CMSSignedData_Create(cms_msg.get());
     if (!cms_sd) {
         return nullptr;
     }
 
-    NSSCMSContentInfo *cms_cinfo = NSS_CMSMessage_GetContentInfo(cms_msg);
+    NSSCMSContentInfo *cms_cinfo = NSS_CMSMessage_GetContentInfo(cms_msg.get());
 
-    if (NSS_CMSContentInfo_SetContent_SignedData(cms_msg, cms_cinfo, cms_sd) != SECSuccess) {
+    if (NSS_CMSContentInfo_SetContent_SignedData(cms_msg.get(), cms_cinfo, cms_sd) != SECSuccess) {
         return nullptr;
     }
 
     cms_cinfo = NSS_CMSSignedData_GetContentInfo(cms_sd);
 
     // Attach NULL data as detached data
-    if (NSS_CMSContentInfo_SetContent_Data(cms_msg, cms_cinfo, nullptr, PR_TRUE) != SECSuccess) {
+    if (NSS_CMSContentInfo_SetContent_Data(cms_msg.get(), cms_cinfo, nullptr, PR_TRUE) != SECSuccess) {
         return nullptr;
     }
 
     // hardcode SHA256 these days...
-    NSSCMSSignerInfo *cms_signer = NSS_CMSSignerInfo_Create(cms_msg, signing_cert, SEC_OID_SHA256);
+    NSSCMSSignerInfo *cms_signer = NSS_CMSSignerInfo_Create(cms_msg.get(), signing_cert, SEC_OID_SHA256);
     if (!cms_signer) {
         return nullptr;
     }
@@ -1117,13 +1065,19 @@ std::unique_ptr<GooString> SignatureHandler::signDetached(const char *password)
         return nullptr;
     }
 
+    struct PLArenaFreeFalse
+    {
+        void operator()(PLArenaPool *arena) { PORT_FreeArena(arena, PR_FALSE); }
+    };
+    std::unique_ptr<PLArenaPool, PLArenaFreeFalse> arena { PORT_NewArena(maxSupportedSignatureSize) };
+
     // Add the signing certificate as a signed attribute.
     ESSCertIDv2 *aCertIDs[2];
     ESSCertIDv2 aCertID;
     // Write ESSCertIDv2.hashAlgorithm.
     aCertID.hashAlgorithm.algorithm.data = nullptr;
     aCertID.hashAlgorithm.parameters.data = nullptr;
-    SECOID_SetAlgorithmID(nullptr, &aCertID.hashAlgorithm, SEC_OID_SHA256, nullptr);
+    SECOID_SetAlgorithmID(arena.get(), &aCertID.hashAlgorithm, SEC_OID_SHA256, nullptr);
 
     // Write ESSCertIDv2.certHash.
     SECItem aCertHashItem;
@@ -1174,7 +1128,7 @@ std::unique_ptr<GooString> SignatureHandler::signDetached(const char *password)
      * { iso(1) member-body(2) us(840) rsadsi(113549) pkcs(1) pkcs9(9)
      *   smime(16) id-aa(2) 47 }
      */
-    if (my_SEC_StringToOID(&aOidData.oid, "1.2.840.113549.1.9.16.2.47", 0) != SECSuccess) {
+    if (my_SEC_StringToOID(arena.get(), &aOidData.oid, "1.2.840.113549.1.9.16.2.47", 0) != SECSuccess) {
         return nullptr;
     }
 
@@ -1193,26 +1147,21 @@ std::unique_ptr<GooString> SignatureHandler::signDetached(const char *password)
     SECItem cms_output;
     cms_output.data = nullptr;
     cms_output.len = 0;
-    PLArenaPool *arena = PORT_NewArena(10000);
 
-    NSSCMSEncoderContext *cms_ecx = NSS_CMSEncoder_Start(cms_msg, nullptr, nullptr, &cms_output, arena, passwordCallback, const_cast<char *>(password), nullptr, nullptr, nullptr, nullptr);
+    NSSCMSEncoderContext *cms_ecx = NSS_CMSEncoder_Start(cms_msg.get(), nullptr, nullptr, &cms_output, arena.get(), passwordCallback, password.empty() ? nullptr : const_cast<char *>(password.c_str()), nullptr, nullptr, nullptr, nullptr);
     if (!cms_ecx) {
-        PORT_FreeArena(arena, PR_FALSE);
         return nullptr;
     }
 
     if (NSS_CMSEncoder_Finish(cms_ecx) != SECSuccess) {
-        PORT_FreeArena(arena, PR_FALSE);
         return nullptr;
     }
 
-    GooString *signature = new GooString(reinterpret_cast<const char *>(cms_output.data), cms_output.len);
+    auto signature = std::make_unique<GooString>(reinterpret_cast<const char *>(cms_output.data), cms_output.len);
 
     SECITEM_FreeItem(pEncodedCertificate, PR_TRUE);
-    NSS_CMSMessage_Destroy(cms_msg);
-    PORT_FreeArena(arena, PR_FALSE);
 
-    return std::unique_ptr<GooString>(signature);
+    return signature;
 }
 
 static char *GetPasswordFunction(PK11SlotInfo *slot, PRBool /*retry*/, void * /*arg*/)
@@ -1266,3 +1215,26 @@ std::vector<std::unique_ptr<X509CertificateInfo>> SignatureHandler::getAvailable
 
     return certsList;
 }
+
+void HashContext::updateHash(unsigned char *data_block, int data_len)
+{
+    HASH_Update(hash_context.get(), data_block, data_len);
+}
+
+std::vector<unsigned char> HashContext::endHash()
+{
+    auto hash_length = digestLength(digest_alg_tag);
+    std::vector<unsigned char> digestBuffer(hash_length);
+    unsigned int result_length = 0;
+    HASH_End(hash_context.get(), digestBuffer.data(), &result_length, digestBuffer.size());
+    digestBuffer.resize(result_length);
+
+    return digestBuffer;
+}
+
+HashContext::HashContext(HashAlgorithm algorithm) : hash_context { HASH_Create(HASH_GetHashTypeByOidTag(ConvertHashAlgorithmToNss(algorithm))) }, digest_alg_tag(algorithm) { }
+
+HashAlgorithm HashContext::getHashAlgorithm() const
+{
+    return digest_alg_tag;
+}
