@@ -6,7 +6,7 @@
 //
 // Copyright 2015 André Guerreiro <aguerreiro1985@gmail.com>
 // Copyright 2015 André Esser <bepandre@hotmail.com>
-// Copyright 2015, 2017-2022 Albert Astals Cid <aacid@kde.org>
+// Copyright 2015, 2017-2023 Albert Astals Cid <aacid@kde.org>
 // Copyright 2016 Markus Kilås <digital@markuspage.com>
 // Copyright 2017, 2019 Hans-Ulrich Jüttner <huj@froreich-bioscientia.de>
 // Copyright 2017, 2019 Adrian Johnson <ajohnson@redneon.com>
@@ -18,6 +18,7 @@
 // Copyright 2021 Theofilos Intzoglou <int.teo@gmail.com>
 // Copyright 2022 Felix Jung <fxjung@posteo.de>
 // Copyright 2022 Erich E. Hoover <erich.e.hoover@gmail.com>
+// Copyright 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 //========================================================================
 
@@ -167,6 +168,7 @@ static const ArgDesc argDesc[] = { { "-nssdir", argGooString, &nssDir, 0, "path
                                    { "-v", argFlag, &printVersion, 0, "print copyright and version info" },
                                    { "-h", argFlag, &printHelp, 0, "print usage information" },
                                    { "-help", argFlag, &printHelp, 0, "print usage information" },
+                                   { "--help", argFlag, &printHelp, 0, "print usage information" },
                                    { "-?", argFlag, &printHelp, 0, "print usage information" },
                                    {} };
 
@@ -354,7 +356,6 @@ int main(int argc, char *argv[])
             return 2;
         }
 
-        const char *pw = (strlen(password) == 0) ? nullptr : password;
         const auto rs = std::unique_ptr<GooString>(reason.toStr().empty() ? nullptr : utf8ToUtf16WithBom(reason.toStr()));
 
         if (newSignatureFieldName.getLength() == 0) {
@@ -370,7 +371,7 @@ int main(int argc, char *argv[])
         }
 
         // We don't provide a way to customize the UI from pdfsig for now
-        const bool success = doc->sign(argv[2], certNickname, pw, newSignatureFieldName.copy(), /*page*/ 1,
+        const bool success = doc->sign(std::string { argv[2] }, std::string { certNickname }, std::string { password }, newSignatureFieldName.copy(), /*page*/ 1,
                                        /*rect */ { 0, 0, 0, 0 }, /*signatureText*/ {}, /*signatureTextLeft*/ {}, /*fontSize */ 0, /*leftFontSize*/ 0,
                                        /*fontColor*/ {}, /*borderWidth*/ 0, /*borderColor*/ {}, /*backgroundColor*/ {}, rs.get(), /* location */ nullptr, /* image path */ "", ownerPW, userPW);
         return success ? 0 : 3;
@@ -416,6 +417,12 @@ int main(int argc, char *argv[])
             return 2;
         }
 
+        if (digestName != std::string("SHA256")) {
+            printf("Only digest SHA256 is supported at the moment\n");
+            printf("Please file a bug report if this is important for you\n");
+            return 2;
+        }
+
         bool getCertsError;
         // We need to call this otherwise NSS spins forever
         getAvailableSigningCertificates(&getCertsError);
@@ -433,14 +440,13 @@ int main(int argc, char *argv[])
         if (etsiCAdESdetached) {
             ffs->setSignatureType(ETSI_CAdES_detached);
         }
-        const char *pw = (strlen(password) == 0) ? nullptr : password;
         const auto rs = std::unique_ptr<GooString>(reason.toStr().empty() ? nullptr : utf8ToUtf16WithBom(reason.toStr()));
         if (ffs->getNumWidgets() != 1) {
             printf("Unexpected number of widgets for the signature: %d\n", ffs->getNumWidgets());
             return 2;
         }
         FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(ffs->getWidget(0));
-        const bool success = fws->signDocument(argv[2], certNickname, digestName, pw, rs.get());
+        const bool success = fws->signDocument(std::string { argv[2] }, std::string { certNickname }, std::string { password }, rs.get());
         return success ? 0 : 3;
     }
 
@@ -456,7 +462,10 @@ int main(int argc, char *argv[])
             for (unsigned int i = 0; i < sigCount; i++) {
                 const bool dumpingOk = dumpSignature(i, sigCount, signatures.at(i), fileName->c_str());
                 if (!dumpingOk) {
-                    return 3;
+                    // for now, do nothing. We have logged a message
+                    // to the user before returning false in dumpSignature
+                    // and it is possible to have "holes" in the signatures
+                    continue;
                 }
             }
             return 0;
@@ -483,30 +492,30 @@ int main(int argc, char *argv[])
         }
 
         const SignatureInfo *sig_info = ffs->validateSignature(!dontVerifyCert, false, -1 /* now */, !noOCSPRevocationCheck, useAIACertFetch);
-        printf("  - Signer Certificate Common Name: %s\n", sig_info->getSignerName());
-        printf("  - Signer full Distinguished Name: %s\n", sig_info->getSubjectDN());
+        printf("  - Signer Certificate Common Name: %s\n", sig_info->getSignerName().c_str());
+        printf("  - Signer full Distinguished Name: %s\n", sig_info->getSubjectDN().c_str());
         printf("  - Signing Time: %s\n", time_str = getReadableTime(sig_info->getSigningTime()));
         printf("  - Signing Hash Algorithm: ");
         switch (sig_info->getHashAlgorithm()) {
-        case HASH_AlgMD2:
+        case HashAlgorithm::Md2:
             printf("MD2\n");
             break;
-        case HASH_AlgMD5:
+        case HashAlgorithm::Md5:
             printf("MD5\n");
             break;
-        case HASH_AlgSHA1:
+        case HashAlgorithm::Sha1:
             printf("SHA1\n");
             break;
-        case HASH_AlgSHA256:
+        case HashAlgorithm::Sha256:
             printf("SHA-256\n");
             break;
-        case HASH_AlgSHA384:
+        case HashAlgorithm::Sha384:
             printf("SHA-384\n");
             break;
-        case HASH_AlgSHA512:
+        case HashAlgorithm::Sha512:
             printf("SHA-512\n");
             break;
-        case HASH_AlgSHA224:
+        case HashAlgorithm::Sha224:
             printf("SHA-224\n");
             break;
         default:
